' This program is provided as Freeware for personal, educational
' and non-commercial use under a GNU General Public License.
'
' Written by Mike Perks (basicx@austin.rr.com), 2005
'
' Robot Electronics TPA81 Infrared Thermal Sensor
' (see http://www.robot-electronics.co.uk/htm/tpa81tech.htm)
'
' Written for ZBasic Application Note AN-212
'
Option Explicit
Option Strict On

Private Const addr As Byte = &HD0 	' default I2C Address
Private Const stopPin as Byte = A.2 ' just used for testing purposes

Private temps(1 to 8) as Byte

' *************************************************************
' Sub Main
' 
' Main tets routine - see PDF file for description and sample output
'**************************************************************
Sub Main()
	Dim i as Byte
	
	' start test
	Debug.Print "Start of TPA81 test"
	Call Sleep(0.25) ' wait for TPA81 to wakeup
	
	' initialize TPA81 and get software version number
	Call InitTPA81(addr, TPA81_UNITS_FAHRENHEIT)
	Debug.Print "Software Version: ";CStr(GetTPA81Version(addr))
	
	'Debug.Print "New address: "; CStrHex(SetTPA81Address(addr, &HD4))

	' scan area using servo to get hottest spot
	' and move servo to that position for rest of measurements
	Debug.Print "Hottest position is "; CStr(findHotSpot())

	' main test loop which gets the temperature readings every second
	' until the stop button is pressed
	Do While GetPin(stopPin) = 1 
		' Print out the ambient temperature and hex value of the temperature flags
		Debug.Print "Ambient Temperature: ";CStr(GetTPA81AmbientTemperature());	
		Debug.Print " Flags: "; CStrHex(GetTPA81TemperatureFlags(25));		' delta is 25F		

		' Print out the temperature array in a row
		Call GetTPA81Temperatures(temps)
		Debug.Print " Temperatures:";
		For i = 1 to 8 
			Debug.Print " ";CStr(temps(i));
		Next
		Debug.Print
		Call Sleep(1.0)
	Loop
	
	' stop using the TPA81
	Call TermTPA81()
	Debug.Print "TPA81 test finished"
End Sub


' *************************************************************
' Function findHotSpot
' 
' Finds the hottest spot using the servo and returns that position
'**************************************************************
Private Function findHotSpot() as Byte
	Dim maxHeat as Integer, heat as Integer
	Dim i as Byte, p as Byte
	
	' set the servo range - this is particular to a servo
	Call SetTPA81ServoRange(1000, 2000)
	
	' get the temperature for each of the 32 servo positions
	maxHeat = 0
	findHotSpot = 0
	For p=0 to 31
		' set the position and wait 40ms for the TPA81 sensor
		Call SetTPA81ServoPosition(p)
		Call Sleep(0.04)
		
		' get the temperature and calculate how hot it is by simply
		' aggregating the eight temperatures
		Call GetTPA81Temperatures(temps)		
		heat = 0		
		For i= 1 to 8
			heat = CInt(temps(i)) + heat
		Next
		
		' see if this is the "hottest" position so far
		If heat > maxHeat Then
			maxHeat = heat
			findHotSpot = p
		End If
	Next
	
	' move the servo to the hottest position
	Call SetTPA81ServoPosition(findHotSpot)
End Function